import maya.cmds as cmds
import maya.mel as mel

try:
	import cPickle as _pickle
except:
	import pickle as _pickle



class ARC_PointCloudViewer:
	@staticmethod
	def setRGBCol(nd, col):
		cmds.setAttr("%s.hiddenInOutliner"%nd, True)
		cmds.setAttr("%s.overrideEnabled"%nd, True)
		cmds.setAttr("%s.overrideRGBColors"%nd, True)
		cmds.setAttr("%s.overrideColorRGB"%nd, col[0], col[1], col[2])
	
	@staticmethod
	def createPoint(name, pos, col):
		pt = cmds.sphere(n="%s_pt" %(name), r=1, po=0, ax=[0,1,0] , nsp=2, s=2, ch=0)[0]
		pt_sh = cmds.listRelatives(pt, children=True)[0]
		shadingGroupPlug = cmds.listConnections("%s.instObjGroups[0]"%pt_sh, p=True)[0]
		ARC_PointCloudViewer.disconnectAttr(shadingGroupPlug)
		ARC_PointCloudViewer.setRGBCol(pt, col)
		cmds.xform(pt, t=pos)
		cmds.xform(pt, cp=True)
		return pt
	
	@staticmethod
	def disconnectAttr(plug):
		try:
			dest = cmds.connectionInfo(plug, sourceFromDestination=True)
			cmds.disconnectAttr(dest, plug)
		except:
			pass
	
	@staticmethod
	def connectAttr(_from, _to):
		cmds.connectAttr(_from, _to, f=True)
		
	@staticmethod
	def loadPointCloud(filename, mayaname):
		grp = cmds.group(n="%s"%mayaname, empty=True)
		cmds.addAttr(grp, ln="pointSize", at="double")
		pntsz = "%s.pointSize"%grp
		cmds.setAttr(pntsz, 0.01, e=True, keyable=True)
		
		try:
			fp = open(filename, "rb")
			data =_pickle.load(fp)
			fp.close()
		except:
			return 0
		
		try:
			for i in range(len(data)):
				d = data[i]
				p = ARC_PointCloudViewer.createPoint("%s_%i"%(mayaname,i), [d[0],d[1],d[2]], [d[3],d[4],d[5]])
				for s in ("sx","sy","sz"):
					cmds.connectAttr(pntsz, "%s.%s"%(p,s), f=True)
				for t in ("tx","ty","tz","rx","ry","rz"):
					cmds.setAttr("%s.%s"%(p,t), lock=True, keyable=False, channelBox=False)
				cmds.parent(p,grp)
		except:
			return -1
		
		return 1
				
	@staticmethod
	def gui():
		wnd = cmds.window("ARC_PCV", title="Load Point Cloud",s=False)
		cmds.rowColumnLayout(numberOfColumns=1)
		cmds.text(label="Select file, set name, press Ok.",font="boldLabelFont",ww=True)
		infile = cmds.textFieldButtonGrp("ARC_PC_infile",label="Point Cloud file:", buttonLabel="...", buttonCommand='cmds.textFieldButtonGrp("ARC_PC_infile", edit=True, text=cmds.fileDialog2(fm=1, okc="Open", ff="Point Cloud Data (.pcd) (*.pcd);;All Files (*.*)")[0])')
		nname = cmds.textFieldGrp("ARC_PC_nname", label='Outliner name:', text='Pointcloud' )
		cmds.separator(h=10, style="in")
		cmds.button(label="Ok", command='ARC_PointCloudViewer.loadPointCloud(cmds.textFieldButtonGrp("ARC_PC_infile", query=True, text=True), cmds.textFieldGrp("ARC_PC_nname", query=True, text=True)), cmds.deleteUI("ARC_PCV")')
		cmds.showWindow(wnd)
		
	@staticmethod
	def setupMenu():
		try:
			mel.eval("global string $gMainWindow;setParent $gMainWindow;menu -label \"Alister\" AlisterMenu;")
		except:
			pass
		try:
			mel.eval("setParent -menu AlisterMenu;menuItem -label \"Point Cloud Viewer\" -command \"python \\\"ARC_PointCloudViewer.gui()\\\"\";")
		except:
			pass